@@ -129,9 +129,13 @@ R_newXMLNode(USER_OBJECT_ name, USER_OBJECT_ attrs, USER_OBJECT_ nameSpace, USER
    xmlNsPtr ns = NULL;
    xmlNodePtr node;
 
-   if(GET_LENGTH(sdoc))
+   if(GET_LENGTH(sdoc)) {
        doc = (xmlDocPtr) R_ExternalPtrAddr(sdoc);
 
+       if(doc->type != XML_DOCUMENT_NODE && doc->type != XML_HTML_DOCUMENT_NODE) 
+	   doc = doc->doc;
+   }
+
     
    if(GET_LENGTH(nameSpace) > 0) {
        /* Need the default namespace and then also any other */
@@ -1029,6 +1033,10 @@ fprintf(stderr, "Creating reference with finalizer for %s (%p) '%s'\n",
 #endif
      R_RegisterCFinalizer(ref, decrementNodeRefCount);
   }
+/*
+#else
+#warning "no ref counting enabled"
+*/
 #endif
   PROTECT(tmp = NEW_CHARACTER(3));
   SET_STRING_ELT(tmp, 0, mkChar(R_getInternalNodeClass(node->type)));
